from django_icons.renderers.icon import IconRenderer


class MaterialRenderer(IconRenderer):
    """
    Render a Material Design icon.

    <i class="material-icons md-light md-inactive">face</i>
    """

    def get_content(self):
        return self.name

    def get_class(self):
        return "material-icons"



SECRET_KEY = "Thanks for using django-icons!"

DEBUG = True

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}}

INSTALLED_APPS = (
    # Default Django apps
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    # Our tests
    "django_icons",
    "tests",
)

ROOT_URLCONF = "tests.app.urls"

MIDDLEWARE = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",  # required for django.contrib.admin
    "django.contrib.messages.middleware.MessageMiddleware",  # required for django.contrib.admin
)

STATIC_URL = "/static/"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.request",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

DJANGO_ICONS = {
    "ICONS": {
        "delete": {"name": "trash", "renderer": "FontAwesome4Renderer"},
        "edit": {"name": "pencil", "title": "Edit", "renderer": "fontawesome4"},
        "feather": {"renderer": "tests.app.renderers.CustomSvgRenderer"},
        "paperplane": {"renderer": "tests.app.renderers.CustomSvgRenderer"},
        "extra-triangle": {
            "name": "fas fa-triangle fa-fw",
            "extra_classes": "extra",
            "renderer": "django_icons.renderers.IconRenderer",
        },
    },
}



from django import template

from ..core import render_icon

register = template.Library()


@register.simple_tag(name="icon")
def icon_tag(name, *args, **kwargs):
    """
    Render an icon.

    This template is an interface to the `render_icon` function from `django_icons`

    **Tag name**::

        icon

    **Parameters**:

        name
            The name of the icon to be rendered

        title
            The title attribute for the icon

            :default: None (no title attribute rendered)

        renderer
            The renderer to use for the icon

            :default: The default renderer as per ``settings.py``, or ultimately `FontAwesome4Renderer`.

    **Usage**::

        {% icon name %}

    **Example**::

        {% icon 'pencil' %}
        {% icon 'pencil' 'fa-big' %}
        {% icon 'trash' title='Delete' %}
    """
    return render_icon(name, *args, **kwargs)



from django.test import TestCase

from .test_template_tags import render_template


class FontAwesome4Test(TestCase):
    """Test the FontAwesome 4 Renderer."""

    def test_icons(self):
        self.assertEqual(
            render_template('{% icon "user" size="lg" renderer="FontAwesome4Renderer" %}'),
            '<i class="fa fa-user fa-lg"></i>',
        )

        # FontAwesome without a mapping, with extra class
        self.assertEqual(
            render_template('{% icon "user" "fa-big" renderer="FontAwesome4Renderer" %}'),
            '<i class="fa fa-user fa-big"></i>',
        )

        # FontAwesome with a mapping
        self.assertEqual(render_template('{% icon "delete" %}'), '<i class="fa fa-trash"></i>')

        # FontAwesome with a mapping, with extra class
        self.assertEqual(
            render_template('{% icon "delete" "foo bar" "end" %}'),
            '<i class="fa fa-trash foo bar end"></i>',
        )

        # FontAwesome with a mapping to a dict with a `title`
        self.assertEqual(
            render_template('{% icon "edit" %}'),
            '<i class="fa fa-pencil" title="Edit"></i>',
        )

        # FontAwesome with a mapping to a dict with a `title`, overwriting the `title` and testing UTF-8 char escaping
        self.assertIn(
            render_template('{% icon "edit" title=title %}', title="Edit'"),
            (
                '<i class="fa fa-pencil" title="Edit&#39;"></i>',
                '<i class="fa fa-pencil" title="Edit&#x27;"></i>',
            ),
        )
        self.assertEqual(
            render_template('{% icon "edit" title=title %}', title='Edit"'),
            '<i class="fa fa-pencil" title="Edit&quot;"></i>',
        )
        self.assertEqual(
            render_template('{% icon "edit" title=title %}', title="<Edit>"),
            '<i class="fa fa-pencil" title="&lt;Edit&gt;"></i>',
        )



from django.test import TestCase

from .test_template_tags import render_template


class MaterialTest(TestCase):
    """Test the Material Design Renderer."""

    def test_icons(self):
        self.assertEqual(
            render_template('{% icon "user" renderer="MaterialRenderer" %}'),
            '<i class="material-icons">user</i>',
        )



from django.test import TestCase

from .test_template_tags import render_template


class Bootstrap3Test(TestCase):
    """Test Bootstrap 3 Renderer."""

    def test_icons(self):
        self.assertEqual(
            render_template('{% icon "user" renderer="Bootstrap3Renderer" %}'),
            '<i class="glyphicon glyphicon-user"></i>',
        )



__all__ = [
    "__version__",
]

try:
    from importlib.metadata import metadata
except ImportError:
    from importlib_metadata import metadata

meta = metadata("django-icons")

__version__ = meta["Version"]



import re
from collections import namedtuple

from django.templatetags.static import static
from django.utils.translation import gettext_lazy as _

from django_icons.renderers.icon import IconRenderer


class ImageRenderer(IconRenderer):
    """
    Render an icon from a local file, each icon being a single file rendered with an <img> tag.

    All the icons must have the same extensions. Variants of an icon can be saved under variants (color, size) of the
    filename, following a pre-defined pattern.

    For each rendered icon, the default CSS classes will be 'icon' and a composed name class 'icon-{name}' where
    {name} is the name of the icon.
    If color and/or size variants are defined, by default composed name classes will be added, following the
    'icon-{variant}' pattern. There will be one class per variant.

    By default, icons are assumed to be located in the 'static/icons' folder. But this can be customized, to have for
    example one folder for each color and/or size.

    For example, to support different colors of an icon, you could postfix the color code as '-c:w' for white or '-c:r'
    for red. Or supporting different sizes like '-s:48' for 48 pixels or '-s:96' for 96 pixels.

    For example, calling in a template:
        {% icon 'myicon' %}

    will produce the following tag:
        <img src="/static/icons/myicon.png'" class="icon icon-myicon" %}">

    To customize the default behaviour, create a custom ``ImageRenderer``::

        # app/renderers.py
        from django_icons.renderers import ImageRenderer

        class CustomImageRenderer(ImageRenderer):
            def get_image_root(cls):
                return "special-icons"

        DJANGO_ICONS = {
            "DEFAULTS": {"renderer": "fontawesome4", "attrs": {"aria-hidden": True}},
            "RENDERERS": {
                "image": "app.renderers.CustomImageRenderer",
            },
            "ICONS": {
                "edit": {"renderer": "image"},
            }
        }

    The template tag supports settings keyword parameters to define the variant attributes and/or set the format. The
    keyword must be the key of the variant attribute. Please note that using keyword parameters will supersede the
    parsing of the name. That is, the variant attributes are set from the keyword parameters and the file name is not
    parsed.
    The format can be defined by the 'format' keyword parameter without altering the definition of variant attributes.
    """

    r"""
    The VariantAttributePattern must contain a named grouped, where the name must be specified as `{}` as it will be
    injected from the key. That is, the pattern used here to match a color specification looks like `-c:(?P<{}>\\w+)`,
    where `-c:` is arbitrary, `<{}>` represents the matching group name and `\\w+` matches for at least one
    alphanumeric character (color code be a name or a code).
    """

    tag = "img"
    format_string = "<{tag}{attrs}>"

    VariantAttributePattern = namedtuple("VariantAttributePattern", ["key", "pattern", "default"])
    _variant_attributes_regex = dict()  # Used to store the compiled regexes of the individual variant attributes

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Dict used to store the variant attributes extracted from icon name
        self.variant_attributes = dict()
        # If variant attributes are set by keyword arguments, they are set here
        # Using keyword arguments supersedes the parsing from the icon name
        vapatterns = self.get_image_variant_attributes_pattern()
        if True in [vap.key in self.kwargs for vap in vapatterns]:
            for vap in vapatterns:
                k = vap.key
                if k in kwargs:
                    self.variant_attributes[k] = kwargs[k]
                elif vap.default:
                    self.variant_attributes[k] = vap.default

    @classmethod
    def get_image_root(cls):
        """
        Return the root path to the images folder.

        By default, returns the path to a 'icons' folder inside the static folder.

        :return: str or callable
        """
        return "icons"

    @classmethod
    def get_image_prefix(cls):
        """
        Return filename prefix.

        For example, Icons8 icons are all prefixed with `icons8-`, so that you can call your icons
        without repeating the prefix in the name.
        """
        return ""

    @classmethod
    def get_image_format(cls):
        """
        Return icon format, without dot.

        For example: 'png'.
        """
        return "png"

    @classmethod
    def get_image_variant_attributes_pattern(cls):
        """
        Return list of patterns to match the available variant attributes.

        :return: list Contains the patterns to match the available variant attributes.
        """
        return [
            cls.VariantAttributePattern("color", r"-c:(?P<{}>\w+)", None),
            cls.VariantAttributePattern("size", r"-s:(?P<{}>\w+)", None),
        ]

    @classmethod
    def _get_image_variant_attributes_regex(cls):
        """
        Compile the regular expression pattern of the variant attributes into regular expression objects.

        :return: dict Key is the variant attribute name and value is a tuple of compiled regex and default value
        """

        if not cls._variant_attributes_regex:
            for v in cls.get_image_variant_attributes_pattern():
                cls._variant_attributes_regex[v.key] = (
                    re.compile(v.pattern.format(v.key)),
                    v.default,
                )
        return cls._variant_attributes_regex

    def get_variant_attributes(self):
        """
        Return the variant attributes.

        :return: dict The variant attributes.
        """
        if not self.variant_attributes:
            for key, pattern in self._get_image_variant_attributes_regex().items():
                regex, default = pattern
                variant = regex.search(self.name)
                if variant:
                    self.variant_attributes[key] = variant.group(key)  # We fetch the matched group by its name
                    self.name = regex.sub("", self.name)  # Remove the parsed variant specifier from the icon name
                elif default:
                    self.variant_attributes[key] = default
        return self.variant_attributes

    def render_variant(self):
        """
        Alter the name of the icon by removing the variant attribute definitions.

        :return: str The variant to be appended to the icon name to build the path to the file in the file system.
        """
        variant_attributes = self.get_variant_attributes()
        variant = ""
        if variant_attributes:
            for v in self.get_image_variant_attributes_pattern():
                if v.key in variant_attributes:
                    variant += "-{}".format(variant_attributes[v.key])
        return variant

    def get_path(self):
        """
        Return the relative path to the icon.

        By default, the icon filename is built as '{name}[-{color}][-{size}][-{variantX}]' where '-{color}' '-{size}'
        and '-{variantX}'s are only added if there are defined (if several are defined, they are added in that order).
        """
        variant = self.render_variant()  # Alters the name
        filename = (
            self.get_image_prefix()
            + self.name
            + variant
            + "."
            + (self.kwargs.get("format", None) or self.get_image_format())
        )
        return "{}/{}".format(static(self.get_image_root()), filename)

    def get_class(self):
        """
        Return CSS class string for the icons.

        First CSS classes: 'icon' and 'icon-{name}' where {name} is the given name of the icon in the templatetag.

        If the icon defines a color and/or size, 'icon-{color}' and/or 'icon-{size}' classes will be added as well.
        """
        css_classes = "icon"
        if self.get_image_prefix():
            css_classes += " icon-{prefix}".format(prefix=self.name)
        for v_p in self.get_image_variant_attributes_pattern():
            if v_p.key in self.get_variant_attributes():
                css_classes += " icon-{variant}-{value}".format(
                    variant=v_p.key, value=self.get_variant_attributes()[v_p.key]
                )
        css_classes += " icon-{name}".format(name=self.name)
        return css_classes

    def get_attrs(self):
        src = self.get_path()  # Alters self.name
        attrs = super(ImageRenderer, self).get_attrs()
        cleaned_name = self.name.replace("-", " ").replace("_", " ").title()
        attrs["alt"] = self.kwargs.get("alt", _("Icon of {}").format(cleaned_name))
        attrs["src"] = src
        return attrs



def _merge_css_item(item):
    """Transform argument into a list with a single string value."""
    if isinstance(item, (list, tuple)):
        return _merge_css_list(*item)
    item = f"{item}" if item else ""
    return [item]


def _merge_css_list(*args):
    """Transform arguments into a list of string values."""
    css_list = []
    for arg in args:
        css_list += _merge_css_item(arg)
    return css_list


def merge_css_list(*args):
    """
    Combine a series of strings and lists into a single list of unique CSS classes.

    Removes duplicates. Gives precedence to first class encountered.
    """
    css_string = " ".join(_merge_css_list(*args))  # Create single string
    css_list = css_string.split()  # Split it to remove all whitespace
    css_list = filter(None, css_list)  # Remove empty strings
    # Remove duplicates, see https://stackoverflow.com/questions/480214/
    return list(dict.fromkeys(css_list))


def merge_css_text(*args):
    """
    Combine a series of strings and lists into one space separated string of unique CSS classes.

    Removes duplicates. Gives precedence to first class encountered.
    """
    return " ".join(merge_css_list(*args))



from copy import deepcopy

from django.conf import settings
from django.utils.module_loading import import_string

from django_icons.css import merge_css_list
from django_icons.renderers import Bootstrap3Renderer, FontAwesome4Renderer, IconRenderer, ImageRenderer
from django_icons.renderers.material import MaterialRenderer

DEFAULT_RENDERERS = {
    "icon": IconRenderer,
    "fontawesome4": FontAwesome4Renderer,
    "bootstrap3": Bootstrap3Renderer,
    "material": MaterialRenderer,
    "image": ImageRenderer,
}


def _get_setting(section, name, default=None):
    """Return a setting from a section, optionally provide default."""

    try:
        # Read from settings
        setting = deepcopy(settings.DJANGO_ICONS[section][name])
    except (AttributeError, KeyError, TypeError):
        # Set to default
        setting = default

    return setting


def get_icon_kwargs_from_settings(name):
    """Return the kwargs from settings, return a dict with at least a `name` key."""

    kwargs_from_settings = _get_setting("ICONS", name, {})
    if isinstance(kwargs_from_settings, str):
        kwargs_from_settings = {"name": kwargs_from_settings}
    kwargs_from_settings.setdefault("name", name)
    return kwargs_from_settings


def get_icon_kwargs(name, *args, **kwargs):
    """Build the kwargs for the icon function based on args and kwargs of the template tag."""

    icon_kwargs = get_icon_kwargs_from_settings(name)
    extra_classes = icon_kwargs.get("extra_classes", "")

    icon_kwargs.update(kwargs)

    extra_classes = merge_css_list(extra_classes, args, kwargs.get("extra_classes", ""))
    if extra_classes:
        icon_kwargs["extra_classes"] = extra_classes

    return icon_kwargs


def _get_icon_renderer_by_name(name):
    """Return class or dotted path to renderer class with this name from dict in settings."""

    default = DEFAULT_RENDERERS.get(name, None)
    renderer_class = _get_setting("RENDERERS", name, default)
    return renderer_class if renderer_class else name


def get_icon_renderer(renderer=None):
    """Return the default icon renderer."""

    renderer_class = renderer or _get_setting("DEFAULTS", "renderer", IconRenderer)

    if isinstance(renderer_class, str):
        renderer_class = _get_icon_renderer_by_name(renderer_class)
        if isinstance(renderer_class, str):
            if "." not in renderer_class:
                renderer_class = f"django_icons.renderers.{renderer_class}"
            renderer_class = import_string(renderer_class)

    return renderer_class



from django.test import TestCase
from django.utils import translation
from django.utils.translation import gettext_lazy as _

from django_icons.core import icon, render_icon
from django_icons.renderers import IconRenderer


class VersionTest(TestCase):
    """Test the package version."""

    def test_version(self):
        from django_icons import __version__

        parts = __version__.split(".")
        self.assertEqual(len(parts), 2)


class RenderIconFunctionTest(TestCase):
    """Test the `render_icon` function."""

    def test_render_icon(self):
        self.assertEqual(
            render_icon("user", title=_("user"), renderer=IconRenderer),
            '<i class="user" title="user"></i>',
        )

    def test_laziness(self):
        user_icon = render_icon("user", title=_("user"), renderer=IconRenderer)
        with translation.override("nl"):
            self.assertEqual(user_icon, '<i class="user" title="gebruiker"></i>')


class IconFunctionTest(TestCase):
    """Test the `icon` function."""

    def test_icon(self):
        with self.assertWarns(DeprecationWarning):
            self.assertEqual(
                icon("user", title=_("user"), renderer=IconRenderer),
                '<i class="user" title="user"></i>',
            )



import warnings

from django.utils.functional import keep_lazy_text

from .utils import get_icon_kwargs, get_icon_renderer


def icon(name, *args, **kwargs):
    """
    Render an icon.

    Deprecated because of frequent name collisions.
    """
    warnings.warn("The `icon` function is deprecated, user `render_icon` instead.", DeprecationWarning)
    return render_icon(name, *args, **kwargs)


@keep_lazy_text
def render_icon(name, *args, **kwargs):
    """
    Render an icon.

    **Parameters**:

        name
            The name of the icon to be rendered

        title
            The title attribute for the icon

            :default: None (no title attribute rendered)

        renderer
            The renderer to use for the icon

            :default: The default renderer as per ``settings.py``, or ultimately `IconRenderer`.

    **Usage**::

        render_icon(name)

    **Example**::

        render_icon('pencil')
        render_icon('trash', title='Delete')
    """
    icon_kwargs = get_icon_kwargs(name, *args, **kwargs)
    renderer_class = get_icon_renderer(icon_kwargs.get("renderer", None))
    renderer = renderer_class(**icon_kwargs)
    return renderer.render()



from django_icons.renderers.icon import IconRenderer


class FontAwesome4Renderer(IconRenderer):
    """
    Render a Font Awesome 4 icon.

    For FontAwesome 5, use IconRenderer.
    """

    def get_size(self):
        size = self.kwargs.get("size", None)
        return f"fa-{size}" if size else ""

    def get_class(self):
        return f"fa fa-{self.name} {self.get_size()}"



from django.forms.utils import flatatt
from django.utils.html import format_html

from django_icons.css import merge_css_list, merge_css_text


class IconRenderer(object):
    """Render an icon as an HTML element."""

    tag = "i"
    format_string = "<{tag}{attrs}>{content}</{tag}>"

    def __init__(self, name, **kwargs):
        """Set name and kwargs."""
        super(IconRenderer, self).__init__()
        self.name = name
        self.content = ""
        self.kwargs = kwargs

    def get_tag(self):
        """Return default tag for HTML builder."""
        return self.tag

    def get_class(self):
        """Return primary CSS class for this icon."""
        return self.name

    def get_extra_classes(self):
        """Return list of other classes for this icon."""
        return merge_css_list(self.kwargs.get("extra_classes", None))

    def get_css_classes(self):
        """Return list of all CSS classes for this icon."""
        return merge_css_list(self.get_class(), self.get_extra_classes())

    def get_attrs(self):
        """Return HTML attributes for this icon."""
        attrs = {}
        title = self.kwargs.get("title")
        if title:
            attrs["title"] = title
        css_classes = merge_css_text(self.get_css_classes())
        if css_classes:
            attrs["class"] = css_classes
        return attrs

    def get_content(self):
        """Return content for the HTML element."""
        return self.content or ""

    def get_format_string(self):
        """Return format string for HTML output."""
        return self.format_string

    def get_format_context(self):
        """Return context for HTML output."""
        return {
            "tag": self.get_tag(),
            "attrs": flatatt(self.get_attrs()),
            "content": self.get_content(),
        }

    def render(self):
        """Return HTML output for icon."""
        return format_html(self.get_format_string(), **self.get_format_context())



import pathlib

from setuptools import find_packages, setup

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()


setup(
    name="django-icons",
    zip_safe=False,
    version="23.1",
    description="Icons for Django",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/zostera/django-icons",
    author="Dylan Verheul",
    author_email="dylan@dyve.net",
    license="BSD-3-Clause",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Framework :: Django :: 4.1",
        "Framework :: Django :: 4.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
    ],
    python_requires=">=3.7",
    install_requires=[
        "Django>=3.2",
        'importlib-metadata<3; python_version<"3.8"',
    ],
)
